/*
 * Author : Christopher Henard (christopher.henard@uni.lu)
 * Date : 21/05/2012
 * Copyright 2012 University of Luxembourg – Interdisciplinary Centre for Security Reliability and Trust (SnT)
 * All rights reserved
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package spl.techniques.ga;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import spl.SPL;
import spl.fm.Product;

/**
 *
 * @author Christopher Henard
 */
public class GA {

    private static Random random;
    private long timeAllowedMS;

    public GA(long timeAllowedMS) {
        this.timeAllowedMS = timeAllowedMS;
        random = new Random();
    }

    public Individual runSimpleGA(int individualSize, int mutateType) throws Exception {
        long startTimeMS = System.currentTimeMillis();
        Individual indiv = new Individual(SPL.getInstance().getUnpredictableProducts(individualSize));
        indiv.fitnessAndOrdering();

        int nbIter = 0;
        while (System.currentTimeMillis() - startTimeMS < timeAllowedMS) {
            Individual newIndiv = new Individual(indiv);
            newIndiv.mutate(mutateType);
            newIndiv.fitnessAndOrdering();
            if (newIndiv.getFitness() > indiv.getFitness()) {
                indiv = newIndiv;
            }
            nbIter++;
        }
        //System.out.println("Iter " + nbIter + " fitness =  " + indiv.getFitness());
        return indiv;
    }
}
